@@ -1,10 +1,7 @@
-from typing import Optional
-
 from .. import Provider as GeoProvider
 
 
 class Provider(GeoProvider):
-
     nationalities = (
         "Afegã",
         "Albanesa",
@@ -123,8 +120,8 @@ class Provider(GeoProvider):
         "Togolesa",
     )
 
-    def nationality(self, min_length: Optional[int] = None, max_length: Optional[int] = None) -> str:
+    def nationality(self) -> str:
         """
         :example: 'Portuguesa'
         """
-        return self.random_element(self.nationalities, min_length, max_length)
+        return self.random_element(self.nationalities)
