@@ -161,20 +161,11 @@ class Provider(BaseProvider):
             raise ValueError("Max value must fit within left digits")
         if left_digits is not None and min_value and math.ceil(math.log10(abs(min_value))) > left_digits:
             raise ValueError("Min value must fit within left digits")
-        if max_value is not None and max_value < -(10**sys.float_info.dig + 1):
-            raise ValueError(f"Max value must be greater than {-10**sys.float_info.dig - 1}")
-        if min_value is not None and min_value > 10**sys.float_info.dig - 1:
-            raise ValueError(f"Min value must be less than {10**sys.float_info.dig - 1}")
 
-        left_digits_requested = left_digits is not None
-        right_digits_requested = right_digits is not None
         # Make sure at least either left or right is set
         if left_digits is None and right_digits is None:
             needed_left_digits = max(1, math.ceil(math.log10(max(abs(max_value or 1), abs(min_value or 1)))))
-            if (sys.float_info.dig - needed_left_digits) < 1:
-                right_digits = 0
-            else:
-                right_digits = self.random_int(1, sys.float_info.dig - needed_left_digits)
+            right_digits = self.random_int(1, sys.float_info.dig - needed_left_digits)
 
         # If only one side is set, choose #digits for other side
         if (left_digits is None) ^ (right_digits is None):
@@ -190,67 +181,32 @@ class Provider(BaseProvider):
                 f"{sys.float_info.dig})",
             )
 
+        sign = ""
         if (min_value is not None) or (max_value is not None):
             # Copy values to ensure we're not modifying the original values and thus going out of bounds
-            temp_min_value = min_value
-            temp_max_value = max_value
+            left_min_value = min_value
+            left_max_value = max_value
             # Make sure left_digits still respected
-            if max_value is None:
-                temp_max_value = 10 ** (left_digits + right_digits)
-                if min_value is not None:
-                    temp_min_value = min_value * 10**right_digits
-            if min_value is None:
-                temp_min_value = -(10 ** (left_digits + right_digits))
-                if max_value is not None:
-                    if max_value < temp_min_value:
-                        temp_min_value = max_value - 1
-                    temp_max_value = max_value * 10**right_digits
+            if left_digits is not None:
+                if max_value is None:
+                    left_max_value = 10**left_digits  # minus smallest representable, adjusted later
+                if min_value is None:
+                    left_min_value = -(10**left_digits)  # plus smallest representable, adjusted later
 
             if max_value is not None and max_value < 0:
-                temp_max_value += 1  # as the random_int will be generated up to max_value - 1
+                left_max_value += 1  # as the random_int will be generated up to max_value - 1
             if min_value is not None and min_value < 0:
-                temp_min_value += 1  # as we then append digits after the left_number
-
-            opposite_signs = (temp_min_value * temp_max_value) < 0
-            if opposite_signs and left_digits_requested:
-                positive_number = self._safe_random_int(
-                    10 ** (left_digits + right_digits - 1),
-                    temp_max_value,
-                    positive=True,
-                )
-                negative_number = self._safe_random_int(
-                    temp_min_value,
-                    -(10 ** (left_digits + right_digits - 1)),
-                    positive=False,
-                )
-                number = self.random_element((positive_number, negative_number))
-            else:
-                number = self._safe_random_int(
-                    temp_min_value,
-                    temp_max_value,
-                    positive,
-                )
-
-            params_requested = (
-                right_digits_requested or left_digits_requested or ((min_value is None) ^ (max_value is None))
+                left_min_value += 1  # as we then append digits after the left_number
+            left_number = self._safe_random_int(
+                left_min_value,
+                left_max_value,
+                positive,
             )
-            if params_requested:
-                result = (
-                    float(f"{str(number)[:-right_digits]}.{str(number)[-right_digits:]}")
-                    if right_digits
-                    else float(number)
-                )
-            else:
-                fix_right_digits_len = right_digits > 0
-                result = float(f"{number}.{self.random_number(right_digits, fix_len=fix_right_digits_len)}")
-
         else:
             sign = "+" if positive else self.random_element(("+", "-"))
-            fix_left_digits_len = left_digits > 0
-            left_number = self.random_number(left_digits, fix_len=fix_left_digits_len)
-            fix_right_digits_len = right_digits > 0
-            result = float(f"{sign}{left_number}.{self.random_number(right_digits, fix_len=fix_right_digits_len)}")
+            left_number = self.random_number(left_digits)
 
+        result = float(f"{sign}{left_number}.{self.random_number(right_digits)}")
         if positive and result == 0:
             if right_digits:
                 result = float("0." + "0" * (right_digits - 1) + "1")
@@ -264,6 +220,17 @@ class Provider(BaseProvider):
             result = min(result, 10**left_digits - 1)
             result = max(result, -(10**left_digits + 1))
 
+        # It's possible for the result to end up > than max_value or < than min_value
+        # When this happens we introduce some variance so we're not always the exactly the min_value or max_value.
+        # Which can happen a lot depending on the difference of the values.
+        # Ensure the variance is bound by the difference between the max and min
+        if max_value is not None:
+            if result > max_value:
+                result = result - (result - max_value + self.generator.random.uniform(0, max_value - min_value))
+        if min_value is not None:
+            if result < min_value:
+                result = result + (min_value - result + self.generator.random.uniform(0, max_value - min_value))
+
         return result
 
     def _safe_random_int(self, min_value: float, max_value: float, positive: bool) -> int:
@@ -291,21 +258,21 @@ class Provider(BaseProvider):
 
     def pydecimal(
         self,
-        left_digits=None,
-        right_digits=None,
-        positive=False,
-        min_value=None,
-        max_value=None,
-    ):
+        left_digits: Optional[int] = None,
+        right_digits: Optional[int] = None,
+        positive: bool = False,
+        min_value: Optional[float] = None,
+        max_value: Optional[float] = None,
+    ) -> Decimal:
         if left_digits is not None and left_digits < 0:
             raise ValueError("A decimal number cannot have less than 0 digits in its " "integer part")
         if right_digits is not None and right_digits < 0:
             raise ValueError("A decimal number cannot have less than 0 digits in its " "fractional part")
         if (left_digits is not None and left_digits == 0) and (right_digits is not None and right_digits == 0):
             raise ValueError("A decimal number cannot have 0 digits in total")
-        if None not in (min_value, max_value) and min_value > max_value:
+        if min_value is not None and max_value is not None and min_value > max_value:
             raise ValueError("Min value cannot be greater than max value")
-        if None not in (min_value, max_value) and min_value == max_value:
+        if min_value is not None and max_value is not None and min_value == max_value:
             raise ValueError("Min and max value cannot be the same")
         if positive and min_value is not None and min_value <= 0:
             raise ValueError("Cannot combine positive=True with negative or zero min_value")
@@ -332,7 +299,7 @@ class Provider(BaseProvider):
 
         if sign == "+":
             if max_value is not None:
-                left_number = str(self.random_int(max(min_value or 0, 0), max_value))
+                left_number = str(self.random_int(int(max(min_value or 0, 0)), int(max_value)))
             else:
                 min_left_digits = math.ceil(math.log10(max(min_value or 1, 1)))
                 if left_digits is None:
@@ -340,7 +307,7 @@ class Provider(BaseProvider):
                 left_number = "".join([str(self.random_digit()) for i in range(0, left_digits)]) or "0"
         else:
             if min_value is not None:
-                left_number = str(self.random_int(max(max_value or 0, 0), abs(min_value)))
+                left_number = str(self.random_int(int(max(max_value or 0, 0)), int(abs(min_value))))
             else:
                 min_left_digits = math.ceil(math.log10(abs(min(max_value or 1, 1))))
                 if left_digits is None:
